@@ -1,15 +1,58 @@
-import jinja2, os, cgi, re, httplib, traceback, json
-import Cookie
+import jinja2, os, re, httplib, traceback, json, sys, StringIO
+import DonkeySimple
 import DonkeySimple.DS as ds
-from _forms import ProcessForm, AnonFormProcessor
-from _auth import Auth
-import settings
-
+from DonkeySimple.DS import get_settings
 sg = ds.SiteGenerator
+settings = get_settings()
+from _forms import ProcessForm, AnonFormProcessor
+from _auth import UserAuth, SecureRequest
+
+from werkzeug.utils import redirect
+from werkzeug.wrappers import Response
+from werkzeug.utils import escape
+from werkzeug.wsgi import SharedDataMiddleware
+# from werkzeug.exceptions import HTTPException, NotFound, BadRequest
+
+
+DEBUG_PORT = 4000
+class SERVER_MODES:
+    UNKNOWN = 0
+    DEBUG = 1
+    CGI = 2
+    WSGI = 3
+SERVER_MODE = SERVER_MODES.UNKNOWN
 
 THIS_PATH = os.path.dirname(os.path.realpath(__file__))
 EDITOR_TEMPLATE_DIR = os.path.join(THIS_PATH, 'templates') 
 
+def make_dev_app():
+    """
+    Generates the DEV server. Here werkzeug takes care of serving all the static
+    files as well as the site itself.
+    SiteServeMiddleware takes care of some server like file serving.
+    """
+    static_files= {'/favicon.ico': os.path.join('static', 'favicon.png'), 
+                   '/static': 'static', 
+                   '/repos': 'repos', 
+                   settings.SITE_URL.rstrip('/'): settings.SITE_PATH}
+    return SiteServeMiddleware(application, static_files, fallback_mimetype='text/html')
+
+def make_cgi_app():
+    """
+    Generates the CGI application, with CGI we can't afford to serve static files with
+    the application. That has to be done by the real server, eg. apache: see the .htaccess
+    file in the site file (the one with settings.py).
+    """
+    return application
+
+@SecureRequest.application
+def application(request):
+    request.check_for_login()
+    view = View(request)
+    response = view.response
+    request.session.save_cookie(response)
+    return response
+
 urls = (
     ('logout$', 'logout'),
     ('add-repo', 'edit_repo'),
@@ -34,57 +77,89 @@ urls = (
 )
 
 class View(object):
-    user = None
-    all_users = None
-    cookie = ''
-    content_type = 'content-type: text/html\n'
-    location = ''
-    response_code = ''
+    site_edit_url = '/'
+    edit_static_url = 'static/'
+    response_code = httplib.OK
     page = ''
+    mimetype = 'text/html'
     
-    def __init__(self):
-        self._msgs = {}
-        self._env = jinja2.Environment(loader=jinja2.FileSystemLoader(EDITOR_TEMPLATE_DIR))
-        self._uri = os.environ['REQUEST_URI']
-        self._site_uri = self._uri[:self._uri.index('/edit/')]
-        if self._site_uri == '': self._site_uri = '/'
-        print 'site_uri:', self._site_uri
-        if hasattr(settings, 'SITE_URI'):
-            set_site_uri = settings.SITE_URI
-            if self._site_uri != set_site_uri:
-                self._add_msg('Auto detected URI does not match settings.SITE_URI: "%s" vs. "%s"'\
-                               % (self._site_uri, set_site_uri), 'warnings')
-        e_index = self._uri.index('/edit/') + 5
-        self._site_edit_uri = self._uri[:e_index] + '/'
-        self._edit_static_uri = join_uri(self._site_edit_uri, 'static/')
-        fields = cgi.FieldStorage()
-        valid_user = self._auth(fields)
-        processing_error = None
+    def __init__(self, request):
+        with CatchStdout(self.print_function):
+            self._msgs = {}
+            self._response = None
+            if SERVER_MODE == SERVER_MODES.UNKNOWN:
+                raise Exception('Mode UNKNOWN')
+            self.request = request
+            if self.request.login_message:
+                self._add_msg(self.request.login_message, ('errors', 'success')[self.request.valid_user])
+            self.isadmin = self.request.valid_user and self.request.user and self.request.user['admin']
+            self._env = jinja2.Environment(loader=jinja2.FileSystemLoader(EDITOR_TEMPLATE_DIR))
+            self.site_url = (settings.SITE_URL, '/')[settings.SITE_URL == '']
+            if SERVER_MODE == SERVER_MODES.CGI:
+                self._uri = request.environ['REQUEST_URI']
+                sim = _string_similar_point(self._uri, request.environ['SCRIPT_NAME'])
+                self.site_edit_url = self._uri[:sim]
+            else:
+                url = request.base_url.replace('http://', '')
+                self._uri = str(url[url.index('/'):])
+            self.edit_static_url = join_uri(self.site_edit_url, 'static/')
+            self._process_forms()
+            self._generate_page(self._uri, loggedin = request.valid_user, error = self.processing_error)
+                  
+    @property
+    def print_function(self):
+        if settings.PRINT_TO == 'PAGE':
+            return self.add_to_page
+        
+    def add_to_page(self, content):
+        if self.mimetype == 'text/html':
+            self.page += content
+    
+    @property
+    def response(self):
+        if self._response is None:
+            self._response = Response(self.page, mimetype = self.mimetype, status = self.response_code)
+        return self._response
+    
+    def _process_forms(self):
+        self.processing_error = None
+        if self.request.method != 'POST':
+            return
+        self.processing_error = None
         try:
-            if valid_user:
-                proc_form = ProcessForm(self._add_msg, fields, self.isadmin, self.username)
+            if self.request.valid_user:
+                proc_form = ProcessForm(self._add_msg, self.request, self.isadmin, self.request.username)
                 if proc_form.regen_users:
-                    valid_user = self._auth(fields)
+                    self.logout(None)
                 self.created_item = proc_form.created_item
             else:
-                AnonFormProcessor(self._add_msg, fields)
+                AnonFormProcessor(self._add_msg, self.request.form)
         except Exception, e:
-            processing_error = e
-        self._generate_page(self._uri, loggedin = valid_user, error = processing_error)
+            print 'processing _process_forms error:', e
+            _, _, exc_traceback = sys.exc_info()
+            self.processing_error = (e, exc_traceback)
         
     def _generate_page(self, uri, loggedin = False, error = None):
-        self.context = {'title': '%s Editor' % settings.SITE_NAME, 'site_name': settings.SITE_NAME, 'site_title': '%s Editor' % settings.SITE_NAME, 
-                        'static_uri': self._edit_static_uri, 'edit_uri': self._site_edit_uri,
-                        'site_uri': self._site_uri, 'json_submit_url': '%ssubmit.json' % self._site_edit_uri}
+        self.context = {'title': '%s Editor' % settings.SITE_NAME, 
+                        'site_name': settings.SITE_NAME, 
+                        'site_title': '%s Editor' % settings.SITE_NAME, 
+                        'static_url': self.edit_static_url, 
+                        'edit_uri': self.site_edit_url,
+                        'site_url': self.site_url, 
+                        'json_submit_url': '%ssubmit.json' % self.site_edit_url, 
+                        'version': DonkeySimple.__version__
+                        }
         if loggedin:
-            self.context.update({'username': self.username, 'admin': self.user['admin']})
+            self.context.update({'username': self.request.username, 'admin': self.isadmin})
+            self.context['anon'] = self.request.is_anon
         self.repo_paths = list(ds.get_all_repos())
         self.context['repos'] = [r for r, _ in self.repo_paths]
         if len(self.context['repos']) == 0:
             self._add_msg('REPOS directory is empty: there are no repos, pages, templates or static files')
         print 'uri: %r' % uri
         if error:
-            self._error_page(error, code = httplib.BAD_REQUEST)
+            e, tb = error
+            self._error_page(e, code = httplib.BAD_REQUEST, tb=tb)
         elif not loggedin:
             if 'reset-password' in uri:
                 self.reset_password()
@@ -92,7 +167,7 @@ class View(object):
                 self.login()
         else:
             found = False
-            for reg, func in urls:
+            for reg, func in urls:  
                 m = re.search(reg, uri)
                 if m:
                     found = True
@@ -102,52 +177,20 @@ class View(object):
                     getattr(self, func)(fid)
                     break
             if not found:
-                if uri != self._site_edit_uri:
+                if uri != self.site_edit_url:
                     self._add_msg('%s not found' % uri, 'errors')
+                    self.response_code = httplib.NOT_FOUND
                 self.index()
         self.context.update(self._msgs)
-#         if hasattr(self, '_static_file'):
-#             self.page = self._static_file
         if hasattr(self, '_template'):
             self.page = self._template.render(**self.context)
-        
-    def _auth(self, fields):
-#         for name in fields:
-#             print '%s:' % name, fields[name].value
-        auth = Auth()
-        if 'username' in fields and 'password' in fields:
-            username = fields['username'].value
-            password = fields['password'].value
-            valid = auth.login(username, password)
-        else:
-            cookies = ''
-            valid = False
-            if 'HTTP_COOKIE' in os.environ:
-                try: cookies = Cookie.SimpleCookie(os.environ["HTTP_COOKIE"])
-                except: auth.msg = 'Error processing cookies'
-            valid = auth.check_cookie(cookies)
-        if not valid:
-            if auth.msg not in ['', None]:
-                self._add_msg(auth.msg, 'errors')
-            return False
-        C = Cookie.SimpleCookie()
-        C[auth.cookie['name']] = auth.cookie['value']
-        for name, value in auth.cookie['extra_values'].items():
-            C[auth.cookie['name']][name] = value
-        self.cookie = C.output()
-        self.user = auth.user
-        self.username = auth.username
-        self.isadmin = self.user['admin']
-        self.all_users = auth.users
-#         self.o_usernames = auth.get_sorted_users()
-        return True
             
     def json_response(self, rid):
         self._json_response(self._msgs)
         
     def _json_response(self, data):
         self._template = self._env.get_template('json_response.json')
-        self.content_type = 'content-type: application/json\n'
+        self.mimetype = 'application/json'
         self.context['json_response'] = json.dumps(data, sort_keys=True, indent=2, separators=(',', ': '))
         
     def login(self):
@@ -159,9 +202,8 @@ class View(object):
         self._template = self._env.get_template('reset_password.html')
         
     def logout(self, fid):
-        auth = Auth()
-        auth.logout(self.username)
-        self.location = 'Location: %s\n' % self._site_edit_uri
+        self.request.logout()
+        self._response = redirect(self.site_edit_url)
     
     def index(self):
         self._template = self._env.get_template('index.html')
@@ -188,15 +230,15 @@ class View(object):
                 self.context['globcon_files'].append({'link': 'edit-globcon-%s' % cf.id, 'name': cf.display})
             if self.isadmin:
                 self.context['users'] = []
-                for u in self.all_users:
+                for u in self.request.users:
                     self.context['users'].append({'link': 'edit-user-%s' % u, 'name': u})
-        except Exception, e:
+        except KeyError, e:
             return self._error_page(e)
         
     def view_repo(self, repo_name):
         self.context['repo_name'] = repo_name
         self.context['title'] = repo_name
-        self.context['action_uri'] = self._site_edit_uri + 'view-repo-' + repo_name
+        self.context['action_uri'] = self.site_edit_url + 'view-repo-' + repo_name
         
         repo_path = (p for r, p in self.repo_paths if r == repo_name).next()
         git_repo = ds.Git(repo_path)
@@ -223,7 +265,7 @@ class View(object):
         t_con = ds.con.Templates()
         self.context['page_templates'] = [cf for _, cf in t_con.cfiles_ordered]
         self.context['other_formats'] = ['markdown', 'html']
-        self.context['action_uri'] = self._site_edit_uri + 'edit-page-last'
+        self.context['action_uri'] = self.site_edit_url + 'edit-page-last'
         if pid is not None:
             page_con = ds.con.Pages()
             try:
@@ -258,7 +300,7 @@ class View(object):
                     if value['type'] == 'string':
                         self.context['page_context_str'].append(con_settings)
                     else:
-                        con_settings['value'] = cgi.escape(value['value'])
+                        con_settings['value'] = escape(value['value'])
                         self.context['page_context_other'].append(con_settings)
             try:
                 self.context['active_page_template_id'] = t_con.get_cfile_name(name = cfile.info['template'], repo = cfile.info['template_repo']).id
@@ -295,7 +337,7 @@ class View(object):
             cfile, template_text = t.get_file_content(fid=tid)
             self.context['file_name'] = cfile.name
             self.context['active_repo'] = cfile.repo
-            self.context['file_text'] = cgi.escape(template_text)
+            self.context['file_text'] = escape(template_text)
         self.context['global_variables'] = generate_global_vars(ds.SiteGenerator().global_context())
         self.context['show_file_text'] = True
         self.context['function'] = 'edit-template'
@@ -312,9 +354,9 @@ class View(object):
             self.context['file_id'] = cfile.id
             self.context['active_repo'] = cfile.repo
             self.context['file_type'] = static.get_file_type(self.context['file_name'])
-            static_uri = join_uri(self._site_edit_uri, ds.REPOS_DIR, cfile.repo, static.DIR, cfile.name)
+            static_uri = join_uri(self.site_edit_url, ds.REPOS_DIR, cfile.repo, static.DIR, cfile.name)
             if self.context['file_type'] == 'Text':
-                self.context['file_text'] = cgi.escape(content)
+                self.context['file_text'] = escape(content)
                 self.context['show_file_text'] = True
             elif self.context['file_type'] == 'Image':
                 self.context['file_image_path'] = static_uri
@@ -350,7 +392,7 @@ class View(object):
         if json_id is not None:
             cfile, libfile_text = con.get_file_content(fid=json_id)
             self.context['active_repo'] = cfile.repo
-            self.context['file_text'] = cgi.escape(libfile_text)
+            self.context['file_text'] = escape(libfile_text)
         self.context['show_file_text'] = True
         self.context['file_name'] = file_name
         self.context['fname_readonly'] = True
@@ -363,20 +405,18 @@ class View(object):
         self.edit_user(uid)
     
     def edit_this_user(self, uid):
-        self.edit_user(self.username)
+        self.edit_user(self.request.username)
     
     def edit_user(self, uid):
         self.context['page_tag'] = 'New User'
         if not self.isadmin:
-            if uid is None:
+            if uid is None or uid != self.request.username or self.request.is_anon:
                 return self._permission_denied()
-            if uid != self.username:
-                return self._permission_denied()
-        self.context['action_uri'] = self._site_edit_uri + 'edit-user-last'
+        self.context['action_uri'] = self.site_edit_url + 'edit-user-last'
         if uid is not None:
             self.context['existing_user'] = True
-            self.context['password_uri'] = self._site_edit_uri + 'set-password-%s' % uid
-            edit_user = self.all_users[uid]
+            self.context['password_uri'] = self.site_edit_url + 'set-password-%s' % uid
+            edit_user = self.request.users[uid]
             self.context['page_tag'] = uid
             self.context['edit_email'] = edit_user['email']
             self.context['edit_username'] = uid
@@ -391,8 +431,8 @@ class View(object):
         self._template = self._env.get_template('edit_user.html')
         
     def set_user_password(self, uid):
-        edit_username = self.all_users[uid]
-        if not self.isadmin and edit_username != self.username:
+        edit_username = self.request.users[uid]
+        if not self.isadmin and edit_username != self.request.username or self.request.is_anon:
             return self._permission_denied()
         self.context['edit_username'] = uid
         self._template = self._env.get_template('set_password.html')
@@ -403,18 +443,22 @@ class View(object):
     def _page_not_found(self):
         self._bad('Page not Found', 'Page not Found: %s' % os.environ['REQUEST_URI'], code=httplib.NOT_FOUND)
         
-    def _error_page(self, e, code = httplib.INTERNAL_SERVER_ERROR):
+    def _error_page(self, e, code = httplib.INTERNAL_SERVER_ERROR, tb = None):
         error_details = None
         if settings.DEBUG:
-            error_details = traceback.format_exc()
+            if tb:
+                error_details = '\n'.join(traceback.format_tb(tb))
+            else:
+                error_details = traceback.format_exc()
         self._bad(str(e),error_details=error_details, code=code)
         
     def _bad(self, error_name, error_details = None, code = httplib.BAD_REQUEST):
+        print error_details
         self.context['error'] = '%d %s' % (code, httplib.responses[code])
-        self.response_code = 'Status: %s\n' % self.context['error']
+        self.response_code = code
         self.context['error_name'] = error_name
         if error_details:
-            self.context['error_details'] = str(error_details)
+#             self.context['error_details'] = str(error_details)
             traceback.print_exc()
         if 'submit.json' in self._uri:
             self._json_response(self.context)
@@ -426,6 +470,89 @@ class View(object):
             self._msgs[mtype] = [msg]
         else:
             self._msgs[mtype].append(msg)
+
+class SiteServeMiddleware(SharedDataMiddleware):
+    
+    def get_directory_loader(self, directory):        
+        def loader(path):
+            """
+            changed from original to serve index.html if the directory is requested
+            and /*.html if if /* is requested.
+            """ 
+            path_orig = path
+            if path is not None:
+                path = os.path.join(directory, path)
+            else:
+                path = directory
+            if os.path.isfile(path):
+                return os.path.basename(path), self._opener(path)
+            
+            attempts = []
+            if path_orig is None:
+                attempts.append('index.html')
+            else:
+                attempts.append(path_orig + '.html')
+            attempts.append('404.html')
+            for attempt_fname in attempts:
+                filepath = os.path.join(directory, attempt_fname)
+                if os.path.isfile(filepath):
+                    return attempt_fname, self._opener(filepath)
+            return None, None
+        return loader
+
+
+class CatchStdout(object):
+    """
+    change working directory and always return
+    """
+    stdout_text = ''
+    override = False
+    
+    def __init__(self, print_to = None):
+        self.print_to = print_to
+    
+    def __enter__(self):
+        if self.override:
+            return
+        sys.stdout.flush()
+        self._stdout_orig = sys.stdout
+        sys.stdout = StringIO.StringIO()
+    
+    def __exit__(self, e_type, e_value, e_tb):
+        if self.override:
+            return
+        self._stdout_orig.flush()
+        self.stdout_text = sys.stdout.getvalue()
+        sys.stdout.close()
+        sys.stdout = self._stdout_orig
+        if settings.DEBUG:
+            if self.print_to:
+                if e_tb:
+                    print self.text
+                else:
+                    self.print_to(self.html)
+            else:
+                print self.text
+        
+    @property
+    def html(self):
+        if self.stdout_text.strip('\r\t\n ') == '':
+            return ''
+        text = ''
+        try:
+            text = escape(self.stdout_text).strip('\r\t\n ')
+        except:
+            pass
+        return '<div class="container"><h4>Debug Output:</h4>\n<pre><code>%s\n</code></pre></div>' % text
+    
+    @property
+    def text(self):
+        if self.stdout_text.strip('\r\t\n ') == '':
+            return ''
+        debug  = '==========DEBUG OUTPUT==========\n'
+        debug += '%s\n' % self.stdout_text
+        debug += '================================'
+        return debug
  
 def get_font_name(filename, add_msg):
     try:
@@ -442,4 +569,16 @@ def join_uri(*args):
     uri = '/' + '/'.join(p.strip('/') for p in args)
     if args[-1].endswith('/'):
         uri += '/'
+    uri = uri.replace('//', '/')
     return uri
+
+    
+def _string_similar_point(s1, s2):
+    i = 0
+    while True:
+        if i == len(s1) or i == len(s2):
+            return i
+        if s1[i] != s2[i]:
+            return i
+        i += 1
+    
\ No newline at end of file
